package eecs4313.demo.mockitoExample;

import static org.junit.Assert.*;

import org.junit.After;
import org.junit.Before;
import org.junit.Ignore;
import org.junit.Test;

import java.util.*;

import org.mockito.*;

public class SimpleServiceTest {

	// Test 1
	@Test
	public void testGetUniqueId() {
		// create mock
		SimpleService test = Mockito.mock(SimpleService.class);

		// define return value for method getUniqueId()
		Mockito.when(test.getUniqueId()).thenReturn(43);

		// use mock in test....
		assertEquals(test.getUniqueId(), 43);
	}

	// Test more than one return value.
	// Demonstrates the return of multiple values
	@Test
	public void testMoreThanOneReturnValue() {
		Iterator i = Mockito.mock(Iterator.class);
		Mockito.when(i.next()).thenReturn("Mockito").thenReturn("is neat!!");
		String result = i.next() + " " + i.next();
		assertEquals("Mockito is neat!!", result);
	}

	// Test return value dependent on method parameter.
	@Test
	public void testReturnValueDependentOnMethodParameter() {
		Comparable c = Mockito.mock(Comparable.class);
		Mockito.when(c.compareTo("Mockito")).thenReturn(1);
		Mockito.when(c.compareTo("Eclipse")).thenReturn(2);
		// assert
		assertEquals(1, c.compareTo("Mockito"));
	}

	/**
	 * Test return value in dependent on method parameter.
	 */
	@Test
	public void testReturnValueInDependentOnMethodParameter() {
		Comparable c = Mockito.mock(Comparable.class);
		Mockito.when(c.compareTo(Mockito.anyInt())).thenReturn(-1);
		assertEquals(-1, c.compareTo(9));
	}

	@Test
	public void testVerify() {
		// create and configure mock
		SimpleService test = Mockito.mock(SimpleService.class);
		Mockito.when(test.getUniqueId()).thenReturn(43);

		// call method testing on the mock with parameter 12
		test.callMe(12);
		test.getUniqueId();
		test.getUniqueId();
		test.invokeSomeMethod("Hello World");
		test.invokeSomeMethod("called at least once");
		test.invokeSomeMethod("called at least twice");
		test.invokeSomeMethod("called five times");
		test.invokeSomeMethod("called at most 3 times");

		// now check if method testing was called with the parameter 12
		Mockito.verify(test).callMe(Matchers.eq(12));

		// was the method called twice?
		Mockito.verify(test, Mockito.times(2)).getUniqueId();

		// other alternatives for verifying the number of method calls for a method
		Mockito.verify(test, Mockito.never()).invokeSomeMethod("never called");
		Mockito.verify(test, Mockito.atLeastOnce()).invokeSomeMethod("called at least once");
		
		//	Will all fail because we didn't met the conditions.
		Mockito.verify(test, Mockito.atLeast(2)).invokeSomeMethod("called at least twice");	 
		Mockito.verify(test, Mockito.times(5)).invokeSomeMethod("called five times");
		Mockito.verify(test, Mockito.atMost(3)).invokeSomeMethod("called at most 3 times");
	}
}
